@@ -337,7 +337,7 @@ public class ResolverImpl implements Resolver
                         try
                         {
                             checkPackageSpaceConsistency(
-                                false, allCandidates.getWrappedHost(revision),
+                                true, allCandidates.getWrappedHost(revision),
                                 allCandidates, revisionPkgMap, new HashMap());
                         }
                         catch (ResolveException ex)
@@ -760,9 +760,6 @@ public class ResolverImpl implements Resolver
             }
             else
             {
-                List<BundleRequirement> reqs = (candCap.getRevision().getWiring() != null)
-                    ? candCap.getRevision().getWiring().getRequirements(null)
-                    : candCap.getRevision().getDeclaredRequirements(null);
                 for (BundleRequirement req
                     : candCap.getRevision().getDeclaredRequirements(null))
                 {
@@ -1211,7 +1208,11 @@ public class ResolverImpl implements Resolver
         Candidates allCandidates, BundleRequirement req, List<Candidates> permutations)
     {
         SortedSet<BundleCapability> candidates = allCandidates.getCandidates(req);
-        if (candidates.size() > 1)
+        // We can only permutate if there is more than one candidate; otherwise,
+        // don't bother. We check for null candidates since during a dynamic import
+        // the importing revision's previously resolved requirements will not have
+        // populated candidates. In all other cases, it should not be null.
+        if ((candidates != null) && (candidates.size() > 1))
         {
             Candidates perm = allCandidates.copy();
             candidates = perm.getCandidates(req);
@@ -1226,7 +1227,11 @@ public class ResolverImpl implements Resolver
         Candidates allCandidates, BundleRequirement req, List<Candidates> permutations)
     {
         SortedSet<BundleCapability> candidates = allCandidates.getCandidates(req);
-        if (candidates.size() > 1)
+        // We can only permutate if there is more than one candidate; otherwise,
+        // don't bother. We check for null candidates since during a dynamic import
+        // the importing revision's previously resolved requirements will not have
+        // populated candidates. In all other cases, it should not be null.
+        if ((candidates != null) && (candidates.size() > 1))
         {
             // Check existing permutations to make sure we haven't
             // already permutated this requirement. This check for
@@ -1394,12 +1399,22 @@ public class ResolverImpl implements Resolver
             List<BundleCapability> caps = (cap.getRevision().getWiring() != null)
                 ? cap.getRevision().getWiring().getCapabilities(null)
                 : cap.getRevision().getDeclaredCapabilities(null);
-            for (int capIdx = 0; capIdx < caps.size(); capIdx++)
+            for (BundleCapability sourceCap : caps)
             {
-                if (caps.get(capIdx).getNamespace().equals(BundleRevision.PACKAGE_NAMESPACE)
-                    && caps.get(capIdx).getAttributes().get(BundleRevision.PACKAGE_NAMESPACE).equals(pkgName))
+                if (sourceCap.getNamespace().equals(BundleRevision.PACKAGE_NAMESPACE)
+                    && sourceCap.getAttributes().get(BundleRevision.PACKAGE_NAMESPACE).equals(pkgName))
                 {
-                    sources.add(caps.get(capIdx));
+                    // Since capabilities may come from fragments, we need to check
+                    // for that case and wrap them.
+                    if (!cap.getRevision().equals(sourceCap.getRevision()))
+                    {
+                        sources.add(
+                            new HostedCapability(cap.getRevision(), (BundleCapabilityImpl) sourceCap));
+                    }
+                    else
+                    {
+                        sources.add(sourceCap);
+                    }
                 }
             }
 
